package ch.papyro.flutter_verifai;

import android.app.Activity;
import android.content.res.Resources;
import android.util.Log;

import androidx.annotation.NonNull;

import com.verifai.core.Verifai;
import com.verifai.core.VerifaiConfiguration;
import com.verifai.core.listeners.VerifaiResultListener;
import com.verifai.core.result.VerifaiResult;

import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

final class MethodCallHandlerImpl implements MethodChannel.MethodCallHandler {
    private final Activity activity;
    private final MethodChannel methodChannel;

    MethodCallHandlerImpl(
            Activity activity,
            BinaryMessenger messenger) {
        this.activity = activity;
        methodChannel = new MethodChannel(messenger, "flutter_verifai");
        methodChannel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        switch (call.method) {
            case "initializeVerifai":

                String licence = call.argument("licence");
                assert licence != null;
                Verifai.setLicence(activity, licence); //

                VerifaiConfiguration configuration = new VerifaiConfiguration();
                configuration.setScanDuration(5.0);
                Verifai.configure(configuration);
                VerifaiResultListener resultListener = new VerifaiResultListener() {
                    @Override
                    public void onCanceled() {
                        Log.println(Log.ERROR, "simon", "canceled");
                    }

                    @Override
                    public void onSuccess(VerifaiResult verifaiResult) {
                        if(verifaiResult.mrzData != null) {
                            VerificationResult verificationResult = new VerificationResult(verifaiResult.mrzData.firstName,
                                    verifaiResult.mrzData.lastName,
                                    verifaiResult.mrzData.dateOfBirth,
                                    verifaiResult.mrzData.mrzString,
                                    verifaiResult.mrzData.sex,
                                    verifaiResult.mrzData.countryCode,
                                    verifaiResult.mrzData.dateOfExpiry);
                            Map<String, Object> resultMap = new HashMap<String,Object>();
                            Field[] allFields = verificationResult.getClass().getDeclaredFields();
                            for (Field field : allFields) {
                                field.setAccessible(true);
                                Object value = null;
                                try {
                                    value = field.get(verificationResult);
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                }
                                resultMap.put(field.getName(), value);
                            }
                            result.success(resultMap);
                        }
                    }

                    @Override
                    public void onError(@NotNull Throwable throwable) {
                        Log.d("error", throwable.getMessage());
                    }
                };
                Verifai.startScan(activity, resultListener);// The licence string that has been obtained from the dashboard.
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    void stopListening() {
        methodChannel.setMethodCallHandler(null);
    }
}
