//
// Created by Bruno Schnenk on 12.08.20.
//

import Foundation
import Verifai
import VerifaiCommons

class VerifaiViewController: UIViewController {

    var licenceString: String = ""

    var delegate: VerifaiViewControllerDelegate?

    override func viewDidLoad() {
        print("viewDidLoad")
        switch VerifaiCommons.setLicence(licenceString) {
            case .success(_):
                print("Successfully configured Verifai")
            case .failure(let error):
                print("🚫 Licence error: \(error)")
        }
        scan()
    }

    private func scan() {
        do {
                try Verifai.start(over: self) { result in
                    switch result {
                    case .failure(let err):
                        self.errorResult(errorMessage: "\(err)")
                    case .success(let result):
                         let data: [String: String?] = [
                            "firstName": result.mrzData?.firstName,
                            "lastName": result.mrzData?.lastName,
                            "dateOfBirth": result.mrzData?.dateOfBirth,
                            "idCode": result.mrzData?.mrzString,
                            "sex": result.mrzData?.sex,
                            "country": result.mrzData?.countryCode,
                            "dateOfExpiry": result.mrzData?.dateOfExpiry
                        ]
                        self.scanResult(data)
                        self.dismiss(animated: false)
                    }
                }
            } catch {
                print("🚫 Licence error: \(error)")
                errorResult(errorMessage: error.localizedDescription)
            }
        }
    
    private func errorResult(errorMessage: String) {
        delegate?.verifaiResultFailed(self, error: errorMessage)
        dismiss(animated: false)
    }

    private func scanResult(_ scanResult: Dictionary<String, String?>) {
        self.delegate?.verifaiResultSuccess(self, scanResult: scanResult)
        dismiss(animated: false)
    }
}